@@ -32,6 +32,7 @@
 
 extern int GetAmplitude(void);
 extern void DoSonicSpeed(int value);
+extern int saved_parameters[];
 
 
 // convert from words-per-minute to internal speed factor
@@ -103,7 +104,7 @@ static unsigned char pause_factor_350[] = {
 15,15,15,15,15};                // 370
 
 // wav_factor adjustments for speeds 350 to 450
-// Use this to calibrate speed for wpm 350-450 
+// Use this to calibrate speed for wpm 350-450
 static unsigned char wav_factor_350[] = {
  120, 121, 120, 119, 119,   // 350
  118, 118, 117, 116, 116,   // 355
@@ -135,6 +136,8 @@ static int speed3 = 118;
 
 //#define TEST_SPEED
 
+#ifdef INCLUDE_SONIC
+
 void SetSpeed(int control)
 {//=======================
 	int x;
@@ -165,7 +168,7 @@ void SetSpeed(int control)
 	{
 		DoSonicSpeed(1 * 1024);
 	}
-	if((wpm_value > 450) || ((wpm_value > speed.fast_settings[0]) && (wpm > 350)))
+	if((wpm_value >= 450) || ((wpm_value > speed.fast_settings[0]) && (wpm > 350)))
 	{
 		wpm2 = wpm;
 		wpm = 175;
@@ -237,6 +240,148 @@ void SetSpeed(int control)
 		}
 	}
 
+	if(control & 2)
+	{
+		// these are used in synthesis file
+
+		if(wpm > 350)
+		{
+			speed.lenmod_factor = 85 - (wpm - 350) / 3;
+			speed.lenmod2_factor = 60 - (wpm - 350) / 8;
+		}
+		else
+		if(wpm > 250)
+		{
+			speed.lenmod_factor = 110 - (wpm - 250)/4;
+			speed.lenmod2_factor = 110 - (wpm - 250)/2;
+		}
+
+		s1 = (x * voice->speedf1)/256;
+
+		if(wpm >= 170)
+			speed.wav_factor = 110 + (150*s1)/128;  // reduced speed adjustment, used for playing recorded sounds
+		else
+			speed.wav_factor = 128 + (128*s1)/130;  // = 215 at 170 wpm
+
+		if(wpm >= 350)
+		{
+			speed.wav_factor = wav_factor_350[wpm-350];
+		}
+
+		if(wpm >= 390)
+		{
+			speed.min_sample_len = 450 - (wpm - 400)/2;
+			if(wpm > 440)
+				speed.min_sample_len = 420 - (wpm - 440);
+		}
+
+// adjust for different sample rates
+speed.min_sample_len = (speed.min_sample_len * samplerate_native) / 22050;
+
+		speed.pause_factor = (256 * s1)/115;      // full speed adjustment, used for pause length
+		speed.clause_pause_factor = 0;
+
+		if(wpm > 430)
+		{
+			speed.pause_factor = 12;
+//			speed.clause_pause_factor = 15;
+		}
+		else
+		if(wpm > 400)
+		{
+			speed.pause_factor = 13;
+//			speed.clause_pause_factor = 15;
+		}
+		else
+		if(wpm > 374)
+		{
+			speed.pause_factor = 14;
+		}
+		else
+		if(wpm > 350)
+		{
+			speed.pause_factor = pause_factor_350[wpm - 350];
+		}
+
+		if(speed.clause_pause_factor == 0)
+		{
+			// restrict the reduction of pauses between clauses
+			if((speed.clause_pause_factor = speed.pause_factor) < 16)
+				speed.clause_pause_factor = 16;
+		}
+	}
+
+#ifdef TEST_SPEED
+//if(control==3)
+printf("%3d: speedf %d %d %d   x=%d  pause=%d %d   wav=%d  lenmod=%d %d\n",wpm,speed1,speed2,speed3, speed_lookup[wpm2-80], speed.pause_factor,speed.clause_pause_factor, speed.wav_factor,speed.lenmod_factor,speed.lenmod2_factor);
+#endif
+}  //  end of SetSpeed
+
+#else  // not using sonic speed-up
+
+void SetSpeed(int control)
+{//=======================
+// This is the earlier version of SetSpeed() before sonic speed-up was added
+	int x;
+	int s1;
+	int wpm;
+	int wpm2;
+
+	speed.loud_consonants = 0;
+	speed.min_sample_len = 450;
+	speed.lenmod_factor = 110;   // controls the effect of FRFLAG_LEN_MOD reduce length change
+	speed.lenmod2_factor = 100;
+
+	wpm = embedded_value[EMBED_S];
+	if(control == 2)
+		wpm = embedded_value[EMBED_S2];
+
+#ifdef TEST_SPEED
+	if(wpm > 1000)
+	{
+		// TESTING
+		test = wpm / 1000;
+		wpm = wpm % 1000;
+	}
+#endif
+
+	if(voice->speed_percent > 0)
+	{
+		wpm = (wpm * voice->speed_percent)/100;
+	}
+	if(wpm > 450)
+		wpm = 450;
+
+	if(wpm > 360)
+	{
+		speed.loud_consonants = (wpm - 360) / 8;
+	}
+
+	wpm2 = wpm;
+	if(wpm > 359) wpm2 = 359;
+	if(wpm < 80) wpm2 = 80;
+	x = speed_lookup[wpm2-80];
+
+	if(wpm >= 380)
+		x = 7;
+	if(wpm >= 400)
+		x = 6;
+
+	if(control & 1)
+	{
+		// set speed factors for different syllable positions within a word
+		// these are used in CalcLengths()
+		speed1 = (x * voice->speedf1)/256;
+		speed2 = (x * voice->speedf2)/256;
+		speed3 = (x * voice->speedf3)/256;
+
+		if(x <= 7)
+		{
+			speed1 = x;
+			speed2 = speed3 = x - 1;
+		}
+	}
+
 	if(control & 2)
 	{
 		// these are used in synthesis file
@@ -311,6 +456,8 @@ printf("%3d: speedf %d %d %d   pause=%d %d   wav=%d  lenmod=%d %d\n",wpm,speed1,
 #endif
 }  //  end of SetSpeed
 
+#endif   // of INCLUDE_SONIC
+
 
 #ifdef deleted
 void SetAmplitude(int amp)
@@ -319,7 +466,7 @@ void SetAmplitude(int amp)
 
 	if((amp >= 0) && (amp <= 20))
 	{
-		option_amplitude = (amplitude_factor[amp] * 480)/256; 
+		option_amplitude = (amplitude_factor[amp] * 480)/256;
 	}
 }
 #endif
@@ -343,6 +490,7 @@ void SetParameter(int parameter, int value, int relative)
 		}
 	}
 	param_stack[0].parameter[parameter] = new_value;
+	saved_parameters[parameter] = new_value;
 
 	switch(parameter)
 	{
@@ -427,6 +575,7 @@ void CalcLengths(Translator *tr)
 	int  last_pitch = 0;
 	int  pitch_start;
 	int  length_mod;
+	int  next2type;
 	int  len;
 	int  env2;
 	int  end_of_clause;
@@ -461,7 +610,7 @@ void CalcLengths(Translator *tr)
 		case phPAUSE:
 			last_pitch = 0;
 			break;
-			
+
 		case phSTOP:
 			last_pitch = 0;
 			if(prev->type == phFRICATIVE)
@@ -541,8 +690,19 @@ void CalcLengths(Translator *tr)
 
 				p->prepause = 40;
 
-				if((prev->type == phPAUSE) || (prev->type == phVOWEL)) // || (prev->ph->mnemonic == ('/'*256+'r')))
-					p->prepause = 0;
+				if(prev->type == phVOWEL)
+				{
+					p->prepause = 0;   // use murmur instead to link from the preceding vowel
+				}
+				else
+				if(prev->type == phPAUSE)
+				{
+					// reduce by the length of the preceding pause
+					if(prev->length < p->prepause)
+						p->prepause -= prev->length;
+					else
+						p->prepause = 0;
+				}
 				else
 				if(p->newword==0)
 				{
@@ -565,7 +725,7 @@ void CalcLengths(Translator *tr)
 			p->amp = tr->stress_amps[0];  // unless changed later
 			p->length = 256;  //  TEMPORARY
 			min_drop = 0;
-			
+
 			if(p->newword)
 			{
 				if(prev->type==phLIQUID)
@@ -588,12 +748,12 @@ void CalcLengths(Translator *tr)
 				if((prev->type==phVOWEL) || (prev->type == phLIQUID))
 				{
 					p->length = prev->length;
-					
+
 					if(p->type == phLIQUID)
 					{
 						p->length = speed1;
 					}
-	
+
 					if(next->type == phVSTOP)
 					{
 						p->length = (p->length * 160)/100;
@@ -679,9 +839,17 @@ if(stress <= 1)
 				next3 = &phoneme_list[ix+4];
 			}
 
+			next2type = next2->ph->length_mod;
 			if(more_syllables==0)
 			{
-				len = tr->langopts.length_mods0[next2->ph->length_mod *10+ next->ph->length_mod];
+				if(next->newword || next2->newword)
+				{
+					// don't use 2nd phoneme over a word boundary, unless it's a pause
+					if(next2type != 1)
+						next2type = 0;
+				}
+
+				len = tr->langopts.length_mods0[next2type *10+ next->ph->length_mod];
 
 				if((next->newword) && (tr->langopts.word_gap & 0x20))
 				{
@@ -693,7 +861,7 @@ if(stress <= 1)
 			}
 			else
 			{
-				length_mod = tr->langopts.length_mods[next2->ph->length_mod *10+ next->ph->length_mod];
+				length_mod = tr->langopts.length_mods[next2type *10+ next->ph->length_mod];
 
 				if((next->type == phNASAL) && (next2->type == phSTOP || next2->type == phVSTOP) && (next3->ph->phflags & phFORTIS))
 					length_mod -= 15;
@@ -744,7 +912,7 @@ if(stress <= 1)
 			{
 				// this is the last syllable in the clause, lengthen it - more for short vowels
 				len = (p->ph->std_length * 2);
-				if(tr->langopts.stress_flags & 0x40000)
+				if(tr->langopts.stress_flags & S_EO_CLAUSE1)
 					len=200;  // don't lengthen short vowels more than long vowels at end-of-clause
 				length_mod = length_mod * (256 + (280 - len)/3)/256;
 			}
@@ -818,11 +986,11 @@ if(p->type != phVOWEL)
 			next->synthflags &= ~SFLAG_SEQCONTINUE;
 			if(next->type == phNASAL && next2->type != phVOWEL)
 				next->synthflags |= SFLAG_SEQCONTINUE;
-				
+
 			if(next->type == phLIQUID)
 			{
 				next->synthflags |= SFLAG_SEQCONTINUE;
-					
+
 				if(next2->type == phVOWEL)
 				{
 					next->synthflags &= ~SFLAG_SEQCONTINUE;
