@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2013 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #ifndef PLATFORM_DOS
 #ifdef PLATFORM_WINDOWS
@@ -86,7 +87,7 @@ static const char *help_text =
 "-p <integer>\n"
 "\t   Pitch adjustment, 0 to 99, default is 50\n"
 "-s <integer>\n"
-"\t   Speed in words per minute, 80 to 450, default is 175\n"
+"\t   Speed in approximate words per minute. The default is 175\n"
 "-v <voice name>\n"
 "\t   Use voice file of this name from espeak-data/voices\n"
 "-w <wave file name>\n"
@@ -109,9 +110,16 @@ static const char *help_text =
 "--punct=\"<characters>\"\n"
 "\t   Speak the names of punctuation characters during speaking.  If\n"
 "\t   =<characters> is omitted, all punctuation is spoken.\n"
-"--split=\"<minutes>\"\n"
+"--sep=<character>\n"
+"\t   Separate phonemes (from -x --ipa) with <character>.\n"
+"\t   Default is space, z means ZWJN character.\n"
+"--split=<minutes>\n"
 "\t   Starts a new WAV file every <minutes>.  Used with -w\n"
 "--stdout   Write speech output to stdout\n"
+"--tie=<character>\n"
+"\t   Use a tie character within multi-letter phoneme names.\n"
+"\t   Default is U+361, z means ZWJ character.\n"
+"--version  Shows version number and date, and location of espeak-data\n"
 "--voices=<language>\n"
 "\t   List the available voices for the specified language.\n"
 "\t   If <language> is omitted, then list all voices.\n";
@@ -159,13 +167,16 @@ void DisplayVoices(FILE *f_out, char *language)
 	const char *p;
 	int len;
 	int count;
+	int c;
+	int j;
 	const espeak_VOICE *v;
 	const char *lang_name;
 	char age_buf[12];
+	char buf[80];
 	const espeak_VOICE **voices;
 	espeak_VOICE voice_select;
 
-	static char genders[4] = {' ','M','F',' '};
+	static char genders[4] = {'-','M','F','-'};
 
 	if((language != NULL) && (language[0] != 0))
 	{
@@ -181,7 +192,7 @@ void DisplayVoices(FILE *f_out, char *language)
 		voices = espeak_ListVoices(NULL);
 	}
 
-	fprintf(f_out,"Pty Language Age/Gender VoiceName       File        Other Langs\n");
+	fprintf(f_out,"Pty Language Age/Gender VoiceName          File          Other Languages\n");
 
 	for(ix=0; (v = voices[ix]) != NULL; ix++)
 	{
@@ -199,8 +210,16 @@ void DisplayVoices(FILE *f_out, char *language)
 
 			if(count==0)
 			{
-				fprintf(f_out,"%2d  %-12s%s%c  %-17s %-11s ",
-               p[0],lang_name,age_buf,genders[v->gender],v->name,v->identifier);
+				for(j=0; j < sizeof(buf); j++)
+				{
+					// replace spaces in the name
+					if((c = v->name[j]) == ' ')
+						c = '_';
+					if((buf[j] = c) == 0)
+						break;
+				}
+				fprintf(f_out,"%2d  %-12s%s%c  %-20s %-13s ",
+               p[0],lang_name,age_buf,genders[v->gender],buf,v->identifier);
 			}
 			else
 			{
@@ -231,16 +250,22 @@ static int OpenWaveFile(const char *path, int rate)
 	if(path == NULL)
 		return(2);
 
-	if(strcmp(path,"stdout")==0)
+	while(isspace(*path)) path++;
+
+	f_wave = NULL;
+	if(path[0] != 0)
 	{
+		if(strcmp(path,"stdout")==0)
+		{
 #ifdef PLATFORM_WINDOWS
 // prevent Windows adding 0x0d before 0x0a bytes
-		_setmode(_fileno(stdout), _O_BINARY);
+			_setmode(_fileno(stdout), _O_BINARY);
 #endif
-		f_wave = stdout;
+			f_wave = stdout;
+		}
+		else
+			f_wave = fopen(path,"wb");
 	}
-	else
-		f_wave = fopen(path,"wb");
 
 	if(f_wave != NULL)
 	{
@@ -397,6 +422,7 @@ static int initialise(void)
 {//========================
 	int param;
 	int result;
+	int srate = 22050;   // default sample rate
 
 	// It seems that the wctype functions don't work until the locale has been set
 	// to something other than the default "C".  Then, not only Latin1 but also the
@@ -412,8 +438,7 @@ static int initialise(void)
 #endif
 
 
-	WavegenInit(22050,0);   // 22050
-	if((result = LoadPhData()) != 1)
+	if((result = LoadPhData(&srate)) != 1)
 	{
 		if(result == -1)
 		{
@@ -423,6 +448,7 @@ static int initialise(void)
 		else
 			fprintf(stderr,"Wrong version of espeak-data 0x%x (expects 0x%x) at %s\n",result,version_phdata,path_home);
 	}
+	WavegenInit(srate,0);
 	LoadConfig();
 	SetVoiceStack(NULL, "");
 	SynthesizeInit();
@@ -434,17 +460,6 @@ static int initialise(void)
 }
 
 
-static void StopSpeak(int unused)
-{//==============================
-	signal(SIGINT,SIG_IGN);
-	// DEBUG
-//	printf("\n*** Interrupting speech output (use Ctrl-D to actually quit).\n");
-	fflush(stdout);
-	SpeakNextClause(NULL,NULL,5);
-	signal(SIGINT,StopSpeak);
-}  //  end of StopSpeak()
-
-
 
 #ifdef NEED_GETOPT
 	struct option {
@@ -484,8 +499,10 @@ int main (int argc, char **argv)
 		{"path",    required_argument, 0, 0x107},
 		{"phonout", required_argument, 0, 0x108},
 		{"pho",     no_argument,       0, 0x109},
-		{"ipa",     no_argument,       0, 0x10a},
+		{"ipa",     optional_argument, 0, 0x10a},
 		{"version", no_argument,       0, 0x10b},
+		{"sep",     optional_argument, 0, 0x10c},
+		{"tie",     optional_argument, 0, 0x10d},
 		{0, 0, 0, 0}
 		};
 
@@ -503,13 +520,17 @@ int main (int argc, char **argv)
 	char *optarg2;
 	int amp = 100;     // default
 	int wordgap = 0;
-	int speaking = 0;
 	int flag_stdin = 0;
 	int flag_compile = 0;
 	int pitch_adjustment = 50;
+	int phoneme_options = 0;
+	int phonemes_separator = 0;
 	espeak_VOICE voice_select;
 	char filename[200];
 	char voicename[40];
+#ifdef USE_PORTAUDIO
+	int speaking = 0;
+#endif
 
 	voicename[0] = 0;
 	mbrola_name[0] = 0;
@@ -610,11 +631,11 @@ int main (int argc, char **argv)
 			break;
 
 		case 'x':
-			option_phonemes = 1;
+			phoneme_options |= espeakPHONEMES_SHOW;
 			break;
 
 		case 'X':
-			option_phonemes = 2;
+			phoneme_options |= espeakPHONEMES_TRACE;
 			break;
 
 		case 'm':
@@ -717,17 +738,56 @@ int main (int argc, char **argv)
 			break;
 
 		case 0x109:  // --pho
-			option_mbrola_phonemes = 16;
+			phoneme_options |= espeakPHONEMES_MBROLA;
 			break;
 
 		case 0x10a:  // --ipa
-			option_phonemes = 3;
+			phoneme_options |= espeakPHONEMES_IPA;
+			if(optarg2 != NULL)
+			{
+				// deprecated and obsolete
+				switch(atoi(optarg2))
+				{
+					case 1:
+						phonemes_separator = '_';
+						break;
+					case 2:
+						phonemes_separator = 0x0361;
+						phoneme_options |= espeakPHONEMES_TIE;
+						break;
+					case 3:
+						phonemes_separator = 0x200d;  // ZWJ
+						phoneme_options |= espeakPHONEMES_TIE;
+						break;
+				}
+				
+			}
 			break;
 
 		case 0x10b:  // --version
 			init_path(argv[0],data_path);
 			printf("speak text-to-speech: %s   Data at: %s\n",version_string,path_home);
 			exit(0);
+			
+		case 0x10c:  // --sep
+			phoneme_options |= espeakPHONEMES_SHOW;
+			if(optarg2 == 0)
+				phonemes_separator = ' ';
+			else
+				utf8_in(&phonemes_separator, optarg2);
+				if(phonemes_separator == 'z')
+					phonemes_separator = 0x200c;  // ZWNJ
+			break;
+
+		case 0x10d:  // --tie
+			phoneme_options |= (espeakPHONEMES_SHOW | espeakPHONEMES_TIE);
+			if(optarg2 == 0)
+				phonemes_separator = 0x0361;   // default: combining-double-inverted-breve
+			else
+				utf8_in(&phonemes_separator, optarg2);
+				if(phonemes_separator == 'z')
+					phonemes_separator = 0x200d;  // ZWJ
+			break;
 
 		default:
 			exit(0);
@@ -737,11 +797,22 @@ int main (int argc, char **argv)
 	init_path(argv[0],data_path);
 	initialise();
 
+	if(voicename[0] == 0)
+		strcpy(voicename,"default");
+
+	if(SetVoiceByName(voicename) != EE_OK)
+	{
+		memset(&voice_select,0,sizeof(voice_select));
+		voice_select.languages = voicename;
+		if(SetVoiceByProperties(&voice_select) != EE_OK)
+		{
+			fprintf(stderr,"%svoice '%s'\n",err_load,voicename);
+			exit(2);
+		}
+	}
 
 	if(flag_compile)
 	{
-		LoadVoice(voicename,5);
-
 #ifdef PLATFORM_DOS
 		char path_dsource[sizeof(path_home)+20];
 		strcpy(path_dsource,path_home);
@@ -763,26 +834,14 @@ int main (int argc, char **argv)
 	}
 
 
-	if(voicename[0] == 0)
-		strcpy(voicename,"default");
-
-	if(SetVoiceByName(voicename) != EE_OK)
-	{
-		memset(&voice_select,0,sizeof(voice_select));
-		voice_select.languages = voicename;
-		if(SetVoiceByProperties(&voice_select) != EE_OK)
-		{
-			fprintf(stderr,"%svoice '%s'\n",err_load,voicename);
-			exit(2);
-		}
-	}
-
 	SetParameter(espeakRATE,speed,0);
 	SetParameter(espeakVOLUME,amp,0);
 	SetParameter(espeakCAPITALS,option_capitals,0);
 	SetParameter(espeakPUNCTUATION,option_punctuation,0);
 	SetParameter(espeakWORDGAP,wordgap,0);
 
+	option_phonemes = phoneme_options | (phonemes_separator << 8);
+
 	if(pitch_adjustment != 50)
 	{
 		SetParameter(espeakPITCH,pitch_adjustment,0);
@@ -869,9 +928,6 @@ int main (int argc, char **argv)
 	}
 	else
 	{
-		// Silence on ^C or SIGINT
-//		signal(SIGINT,StopSpeak);
-
 		// output sound using portaudio
 		WavegenInitSound();
 
